#!/bin/bash

DBP_HOME=${DBP_HOME:-"$HOME/.dbp_get_sh"}

createDefaultConfig() {
	if [ ! -d "$DBP_HOME" ];then
		mkdir "$DBP_HOME";
	fi
	if [ ! -e "$DBP_HOME/source.list" ];then
		cat >"$DBP_HOME/source.list" <<ENDCFG
dbp https://pyra-handheld.com/repo opensource proprietary needdata
ENDCFG
	fi
	if [ ! -e "$DBP_HOME/config" ];then
		cat >"$DBP_HOME/config" <<ENDCFG
target=/dragonbox/packages/
ENDCFG
	fi
}

getRepoInfo() {
	gawk '$1=="dbp"{x=$2;gsub(".*//","",x);gsub("/","-",x);$1="";print x$0}' "$DBP_HOME/source.list"
}
getTarget() {
	gawk -F= '$1=="target"{print $2}' "$DBP_HOME/config"
}
getPackageInfo() {
	gawk -F= -v pck=$2 'BEGIN{p=0}$0=="["pck"]"{p=1}/^$/{p=0}p==1&&/'"$1"'=/{print $2;exit}' "$DBP_HOME/repos/$3/$4/Packages"
}
getPackagesInfo() {
	gawk -F= 'BEGIN{p=""}/^\[/{p=$1;gsub("^.","",p);gsub(".$","",p)}/^'"$1"'=/{print p"\t"$2}' "$DBP_HOME/repos/$2/$3/Packages"
}
getFileMD5() {
	md5sum <"$1"|awk '{print $1}'
}

update() {
	[ ! -e "$DBP_HOME/source.list" ] && createDefaultConfig
	getRepoInfo|while read id url comps;do
		for c in $comps;do
			mkdir -p "$DBP_HOME/repos/$id/$c"
			echo -n "Updating: $id $c..."
			wget -o /dev/null -O "$DBP_HOME/repos/$id/$c/Packages" "$url/dists/dbprepo/$c/Packages"
			touch "$DBP_HOME/repos/$id/$c/Packages"
			echo " done"
		done
	done
}

precheck() {
	[ ! -e "$DBP_HOME/source.list" ] && update
	ok=0
	getRepoInfo|{ while read id url comps;do
		for c in $comps;do
			[ ! -e "$DBP_HOME/repos/$id/$c/Packages" ] && ok=1
			[ "$DBP_HOME/source.list" -nt "$DBP_HOME/repos/$id/$c/Packages" ] && ok=1
		done
	done
	if [ "$ok" -eq 1 ];then
		update
	fi
	}
}

list() {
	precheck
	getRepoInfo|while read id url comps;do
		for c in $comps;do
 			getPackagesInfo "Name" "$id" "$c"|while read id desc;do
				printf "%s\033[30G%s\n" "$id" "$desc"
			done
		done
	done
}

install() {
	pck=$1
	precheck
	getRepoInfo|{ while read id url comps;do
		for c in $comps;do
			path=$(getPackageInfo "Download" "$pck" "$id" "$c")
			if [[ "$path" != "" ]];then
				deps=$(getPackageInfo "Dependency\[dbp\]" "$1"  "$id" "$c")
				if [[ "$deps" != "" ]];then
					echo $deps|sed 's/;/\n/'|while read l;do
						install "$l"
					done
				fi
				md=$(getPackageInfo "MD5sum" "$1"  "$id" "$c")
				fname="${pck}.dbp"
				echo -n "Downloading $fname..."
				wget -o /dev/null -O "$(getTarget)${fname}_new" "$(echo $url|awk -F/ '{print $1"//"$3}')$path"
				echo " done"
				echo -n "Checking..."
				if [[ $(getFileMD5 "$(getTarget)/${fname}_new") == "$md" ]];then
					echo " ok"
					mv "$(getTarget)/${fname}_new" "$(getTarget)/${fname}"
				else
					echo " failed";
					rm "$(getTarget)/${fname}_new"
				fi
				exit;
			fi
		done
	done
	echo "Package \"$pck\" not found"
	}
}

install_all() {
	while [ $# -gt 0 ];do 
		install $1
		shift
	done
}

list_installed() {
	precheck
	getRepoInfo|while read id url comps;do
		for c in $comps;do
 			getPackagesInfo "MD5sum" "$id" "$c"|while read pck md;do
				if [ -e "$(getTarget)/${pck}.dbp" ];then
					printf "%s\033[30G%s\n" "$pck" "$(getPackageInfo "Name"  "$pck" "$id" "$c")"
				fi
			done
		done
	done
}

remove() {
	while [ $# -gt 0 ];do 
		pck=$1
		if [ -e "$(getTarget)/${pck}.dbp" ];then
			echo "Uninstalling $pck"
			rm -f "$(getTarget)/${pck}.dbp"
		fi
		shift
	done
}

list_upgrade() {
	precheck
	getRepoInfo|while read id url comps;do
		for c in $comps;do
 			getPackagesInfo "MD5sum" "$id" "$c"|while read pck md;do
				if [ -e "$(getTarget)/${pck}.dbp" ];then
					if [[ $(getFileMD5 "$(getTarget)/${pck}.dbp") != "$md" ]];then
						printf "%s\033[30G%s\n" "$pck" "$(getPackageInfo "Name"  "$pck" "$id" "$c")"
					fi
				fi
			done
		done
	done
}

upgrade() {
	precheck
	getRepoInfo|while read id url comps;do
		for c in $comps;do
 			getPackagesInfo "MD5sum" "$id" "$c"|while read pck md;do
				if [ -e "$(getTarget)/${pck}.dbp" ];then
					if [[ $(getFileMD5 "$(getTarget)/${pck}.dbp") != "$md" ]];then
						install "$pck"
					fi
				fi
			done
		done
	done
}

showHelp() {
	cat <<ENDHELP
$1 {install|list|update|list|list_installed|list_upgrade|upgrade|remove}
  install <pck>	: Install given package
  remove  <pck> : Remove given package
  update	: Update known package lists
  list		: List all availables packages
  list_installed: List all installed packages
  list_upgrade	: List all upgradables packages
  upgrade	: Upgrade known package
ENDHELP
}

main() {
	case "$1" in
	update)		update;;
	list)		list;;
	list_upgrade)	list_upgrade;;
	list_installed)	list_installed;;
	upgrade)	upgrade;;
	install)	shift;install_all "$@";;
	remove)		shift;remove "$@";;
	help|*)		showHelp "$0";;
	esac
}

main "$@"
